import 'package:clean_framework/src/core/core.dart';
import 'package:logger/logger.dart';

/// The class to observe failures, route changes and other events.
class CleanFrameworkObserver {
  /// Default constructor.
  CleanFrameworkObserver({
    this.enableNetworkLogs = true,
    Logger? logger,
  }) : logger = logger ?? Logger();

  final Logger logger;

  /// Enables network logs.
  final bool enableNetworkLogs;

  /// Default instance of [CleanFrameworkObserver].
  ///
  /// This can be changed in following way:
  /// ```dart
  /// CleanFrameworkObserver.instance = SubClassOfCleanFrameworkObserver();
  /// ```
  static CleanFrameworkObserver instance = CleanFrameworkObserver();

  /// Called when an [error] is thrown by [ExternalInterface]
  /// for the given [request].
  void onExternalError(
    Object externalInterface,
    Request request,
    Object error,
    StackTrace stackTrace,
  ) {
    logger.e(
      'Error occurred while requesting "${request.runtimeType}" '
      'for "${externalInterface.runtimeType}"',
      error: error,
      stackTrace: stackTrace,
    );
  }

  /// Called when a [failureResponse] occurs for the given [request].
  void onFailureResponse(
    Object externalInterface,
    Request request,
    FailureResponse failureResponse,
  ) {}

  /// Called when a success [input] occurs in an use case.
  void onSuccessInput(
    UseCase useCase,
    Output gatewayOutput,
    SuccessInput input,
  ) {
    logger.d(
      '[${useCase.runtimeType}] $gatewayOutput\n' '[Success] $input',
      stackTrace: StackTrace.empty,
    );
  }

  /// Called when a failure [input] occurs in an use case.
  void onFailureInput(
    UseCase useCase,
    Output gatewayOutput,
    FailureInput input,
  ) {
    logger.d(
      '[${useCase.runtimeType}] $gatewayOutput\n' '[Failure] ${input.message}',
      stackTrace: StackTrace.empty,
    );
  }

  /// Called when [location] of the route changes.
  void onLocationChanged(String location) {}
}
